﻿using System;
using Gadgeteer.Interfaces.Fakes.Framework;
using System.Threading;

namespace Gadgeteer.Interfaces.Fakes
{
    public class FakeButton : DelayedAction, IButton
    {
        private static int unNamedButtons = 0;
        private DateTime pressedUntil = DateTime.MinValue;
        private bool isPressed;

        public FakeButton()
            : this("Fake " + ++unNamedButtons)
        {
        }

        public FakeButton(string name)
            : base(name)
        {
            isPressed = false;
        }

        public bool IsPressed
        {
            get { return isPressed; }
        }

        private static readonly TimeSpan defaultPressLength = new TimeSpan(0, 0, 0, 0, 50);
        public void Press()
        {
            Press(defaultPressLength);
        }

        public void Press(TimeSpan pressFor)
        {
            ExecuteAsDelayed(() =>
            {
                OnButtonPressed();
                isPressed = true;

                Thread.Sleep((int)pressFor.Milliseconds);

                isPressed = false;
                OnButtonReleased();
            },
                new TimeSpan(0, 0, 0, 0, 10)
            );
        }

        #region Press/Release events
        private void OnButtonPressed()
        {
            var ev = ButtonPressed;
            if (ev != null)
                ev(this, ButtonState.Pressed);
        }

        public event ButtonEventHandler ButtonPressed;

        private void OnButtonReleased()
        {
            var ev = ButtonReleased;
            if (ev != null)
                ev(this, ButtonState.Released);
        }

        public event ButtonEventHandler ButtonReleased;
        #endregion
    }
}
